/*
 * gsampler-session.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <glib/gi18n-lib.h>

#include "gsampler-session.h"
#include "gsampler-window.h"
#include "gsampler-option.h"
#include "gsampler-utils.h"
#include "gsampler-device-manager.h"
#include "gsampler-device.h"
#include "gsampler-channel-manager.h"
#include "gsampler-channel.h"
#include "gsampler-midi-map.h"
#include "gsampler-client-device.h"
#include "gsampler-client-channel.h"
#include "gsampler-client-midimap.h"
#include "gsampler-client-database.h"

G_DEFINE_TYPE (GsamplerSession, gsampler_session, G_TYPE_OBJECT);

enum  {
	GSAMPLER_SESSION_DUMMY_PROPERTY,
	GSAMPLER_SESSION_NAME,
	GSAMPLER_SESSION_FILENAME,
	GSAMPLER_SESSION_IS_UPDATED,
	GSAMPLER_SESSION_LAST
};

static GParamSpec *ses_pops[GSAMPLER_SESSION_LAST];

const gchar* 
gsampler_session_get_name (GsamplerSession* self) {
	const gchar* result;
	g_return_val_if_fail (self != NULL, NULL);

	result = self->priv->_name;
	return result;
}


static void 
gsampler_session_set_name (GsamplerSession* self, const gchar* value) {
	g_return_if_fail (self != NULL);
	
	g_free (self->priv->_name);
	self->priv->_name = g_strdup (value);
	g_object_notify ((GObject *) self, "name");
}


const gchar* 
gsampler_session_get_filename (GsamplerSession* self) {
	const gchar* result;
	g_return_val_if_fail (self != NULL, NULL);
	
	result = self->priv->_filename;
	return result;
}


static void 
gsampler_session_set_filename (GsamplerSession* self, const gchar* value) {
	g_return_if_fail (self != NULL);

	g_free (self->priv->_filename);
	self->priv->_filename = g_strdup (value);
	g_object_notify ((GObject *) self, "filename");
}


gboolean 
gsampler_session_get_is_updated (GsamplerSession* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	
	result = self->priv->_is_updated;
	return result;
}


static void 
gsampler_session_set_is_updated (GsamplerSession* self, gboolean value) {
	g_return_if_fail (self != NULL);
	
	self->priv->_is_updated = value;
	g_object_notify_by_pspec ((GObject *) self, ses_pops[GSAMPLER_SESSION_IS_UPDATED]);
}

static gboolean 
gsampler_session_save_session_file (GsamplerSession* self) {
	gboolean result = FALSE;
	GFile* file = NULL;
	GFileOutputStream* file_stream = NULL;
	GDataOutputStream* data_stream = NULL;
	GString* buffer;
	GSList* audios = NULL;
	GSList* midis = NULL;
	GSList* channels = NULL;
	GSList* maps = NULL;
	GsamplerClient* client;
	gdouble vol;
	gchar* tmp_name;
	GError * error = NULL;
	g_return_val_if_fail (self != NULL, FALSE);

	client = gsampler_client_get_instance ();
	buffer = g_string_new ("");
	buffer = g_string_append (buffer, "#Linuxsampler LSCP script for Gsampler\n\n");
	buffer = g_string_append (buffer, "RESET\n\n");
	audios = gsampler_device_manager_get_audios ();
	while (audios) {
		GsamplerDevice* device;
		GsamplerDeviceInfo *info = NULL;

		device = (GsamplerDevice*) ((struct Struct_list_widget*) (audios->data))->Sdata;
		info = gsampler_device_get_info (device);
		g_string_append_printf (buffer, "#Audio Device %i\n",info->id);
		g_string_append_printf (buffer, "CREATE AUDIO_OUTPUT_DEVICE %s ",info->driver);
		g_string_append_printf (buffer, "ACTIVE=%s ",bool_to_string (info->active));
		g_string_append_printf (buffer, "CHANNELS=%i ",info->param);
		g_string_append_printf (buffer, "NAME='%s' ",info->name);
		g_string_append_printf (buffer, "SAMPLERATE=%s\n",double_to_string (info->samplerate));

		gint ichan;
		for (ichan = 0; ichan < info->param; ichan++)
		{
			GSList *list = NULL;
			list = gsampler_client_get_avail_device_parameter_bindings (client, info->type,
			                                                            info->id, ichan);
			for (; list; list = list->next) 
			{
				GsamplerDeviceBinding* binding;
				binding = (GsamplerDeviceBinding*) list->data;
				if (binding->active == TRUE) {
					g_string_append_printf (buffer, "SET AUDIO_OUTPUT_CHANNEL_PARAMETER %i ", info->id);
					g_string_append_printf (buffer, "%i %s='%s'\n", ichan,
					                    		binding->type, binding->name);
				}
				gsampler_device_binding_free (binding);
			}
		}
		gsampler_device_info_free (info);
		buffer = g_string_append (buffer, "\n");
		audios = g_slist_next(audios);
	}
	
	midis = gsampler_device_manager_get_midis ();
	while (midis) {
		GsamplerDevice* device;
		GsamplerDeviceInfo *info = NULL;

		device = (GsamplerDevice*) ((struct Struct_list_widget*) (midis->data))->Sdata;
		info = gsampler_device_get_info (device);
		g_string_append_printf (buffer, "#Midi Device %i\n",info->id);
		g_string_append_printf (buffer, "CREATE MIDI_INPUT_DEVICE %s ",info->driver);
		g_string_append_printf (buffer, "ACTIVE=%s ",bool_to_string (info->active));
		g_string_append_printf (buffer, "PORTS=%i ",info->param);
		g_string_append_printf (buffer, "NAME='%s'\n",info->name);

		gint ichan;
		for (ichan = 0; ichan < info->param; ichan++)
		{
			GSList *list = NULL;
			list = gsampler_client_get_avail_device_parameter_bindings (client, info->type,
			                                                            info->id, ichan);
			for (; list; list = list->next) 
			{
				GsamplerDeviceBinding* binding;
				binding = (GsamplerDeviceBinding*) list->data;
				if (binding->active == TRUE) {
					g_string_append_printf (buffer, "SET MIDI_INPUT_PORT_PARAMETER %i ", info->id);
					g_string_append_printf (buffer, "%i %s='%s'\n", ichan,
					                    		binding->type, binding->name);
				}
				gsampler_device_binding_free (binding);
			}
		}
		gsampler_device_info_free (info);
		buffer = g_string_append (buffer, "\n");
		midis = g_slist_next(midis);
	}
	
	channels = gsampler_channel_manager_get_channels ();
	while (channels) {
		GsamplerChannel *channel;
		GsamplerChannelInfo *info = NULL;

		channel = (GsamplerChannel*) ((struct Struct_list_widget*) (channels->data))->Sdata;
		info = gsampler_channel_get_info (channel);
		g_string_append_printf (buffer, "#Channel %i\n",info->id);
		buffer = g_string_append (buffer, "ADD CHANNEL \n");
		g_string_append_printf (buffer, "SET CHANNEL AUDIO_OUTPUT_DEVICE %i %i\n",info->id, info->audio);
		g_string_append_printf (buffer, "SET CHANNEL MIDI_INPUT_DEVICE %i %i\n",info->id, info->midi);
		g_string_append_printf (buffer, "SET CHANNEL MIDI_INPUT_PORT %i %i\n",info->id, info->midi_port);
		g_string_append_printf (buffer, "SET CHANNEL MIDI_INPUT_CHANNEL %i %s\n",info->id, info->midi_chan);
		g_string_append_printf (buffer, "LOAD ENGINE %s %i\n",info->engine, info->id);
		g_string_append_printf (buffer, "LOAD INSTRUMENT NON_MODAL '%s' %i %i\n",info->file, info->instrument, info->id);
		g_string_append_printf (buffer, "SET CHANNEL VOLUME %i %i\n",info->id, (int) info->volume);
		buffer = g_string_append (buffer, "\n");
		gsampler_channel_info_free (info);
		channels = g_slist_next(channels);
	}

	maps = gsampler_midi_map_get_midi_maps ();
	while (maps) {
		GSList* insts;
		gchar* map_name;
		struct Struct_list_widget* list_widget = NULL;
		gint map_key;

		list_widget = (struct Struct_list_widget*) maps->data;
		map_name = (gchar*) list_widget->Sdata;
		map_key = list_widget->Sid;
		g_string_append_printf (buffer, "#Midi-Map %s\n", map_name);
		g_string_append_printf (buffer, "ADD MIDI_INSTRUMENT_MAP '%s'\n", map_name);
		insts = gsampler_client_get_available_midi_instruments (client, map_key);
		while (insts) {
			gchar* inst;
			inst = g_strdup (insts->data);
			if (inst != NULL && g_strcmp0 (inst, "") != 0) {
				gchar** tmp;
				GsamplerMidiInstInfo* info;
				
				inst = string_replace (inst, "{", "");
				inst = string_replace (inst, "}", "");
				if (g_str_has_prefix (inst, ",")) {
					inst = string_slice (inst, 1, strlen (inst));
				}
				tmp = g_strsplit (inst, ",", 0);
				info = gsampler_client_get_midi_instruments_info (client, atoi (tmp[0]), atoi (tmp[1]), atoi (tmp[2]));
				g_string_append_printf (buffer, "MAP MIDI_INSTRUMENT %s %s %s ", tmp[0], tmp[1], tmp[2]);
				g_string_append_printf (buffer, "%s '%s' %i %i", info->engine, info->file, info->inst_nb, (int) info->vol);
				if (info->load_mode != NULL) {
					g_string_append_printf (buffer, " %s", info->load_mode);
				}
				if (info->name != NULL) {
					g_string_append_printf (buffer, " '%s'", info->name);
				}
				buffer = g_string_append (buffer, "\n");
				gsampler_midi_inst_info_free (info);
				g_strfreev (tmp);
			}
			g_free (inst);
			insts = insts->next;
		}
		buffer = g_string_append (buffer, "\n");
		g_slist_free_full (insts, g_free);
		maps = g_slist_next(maps);
	}
	
	vol = gsampler_main_window_get_master_volume ();
	buffer = g_string_append (buffer, "#Master Volume\n");
	g_string_append_printf (buffer, "SET VOLUME %s\n", double_to_string (vol));

	file = g_file_new_for_path (self->priv->_filename);
	tmp_name = string_replace (g_file_get_basename (file), ".lscp", "");
	gsampler_session_set_name (self, tmp_name);
	if (!g_file_query_exists (file, NULL)) {
		file_stream = g_file_create (file, G_FILE_CREATE_NONE, NULL, &error);
		if (error != NULL) {
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			g_error_free (error);
		}
		
		data_stream = g_data_output_stream_new ((GOutputStream*) file_stream);
		g_data_output_stream_put_string (data_stream, buffer->str, NULL, &error);
		if (error != NULL) {
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			gsampler_main_window_send_message ("warning", 
			                                   g_strdup_printf ("Can't create file : %s", self->priv->_filename));
			g_error_free (error);
		}
	} else {
		g_file_replace_contents (file, buffer->str, buffer->len, NULL, FALSE, G_FILE_CREATE_NONE, NULL, NULL, &error);
		if (error != NULL) {
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			gsampler_main_window_send_message ("warning", 
			                                   g_strdup_printf ("Can't create file : %s", self->priv->_filename));
			g_error_free (error);
		}
	}
	gsampler_session_set_is_updated (self, FALSE);
	result = TRUE;
	g_object_unref (client);
	g_string_free (buffer, TRUE);
	g_object_unref (data_stream);
	g_object_unref (file_stream);
	g_object_unref (file);
	return result;
}


static gboolean 
gsampler_session_open_session_file (gpointer data) {
	GFile* file = NULL;
	GDataInputStream* in_stream = NULL;
	GsamplerClient* client;
	GFileInputStream* file_stream = NULL;
	gchar* line;
	GError * error = NULL;
	GsamplerMainWindow *win = NULL;
	GsamplerSession* self = GSAMPLER_SESSION (data);
	g_return_val_if_fail (self != NULL, FALSE);
	
  	if (!g_source_is_destroyed (g_main_current_source ()))
    {
		gsampler_main_window_set_wait_cursor (TRUE);
		client = gsampler_client_get_instance ();
		file = g_file_new_for_path (self->priv->_filename);

		file_stream = g_file_read (file, NULL, &error);
		if (error != NULL) {
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			g_object_unref (file_stream);
			g_object_unref (file);
			g_error_free (error);
			return FALSE;
		}
		in_stream = g_data_input_stream_new ((GInputStream*) file_stream);
		g_data_input_stream_set_newline_type (in_stream, G_DATA_STREAM_NEWLINE_TYPE_ANY);
		if (!g_file_query_exists (file, NULL)) {
			g_warning ("File '%s' doesn't exist.", g_file_get_path (file));
			g_object_unref (client);
			g_object_unref (in_stream);
			g_object_unref (file_stream);
			g_object_unref (file);
			g_error_free (error);
			return FALSE;
		}
		line = g_data_input_stream_read_line (in_stream, NULL, NULL, &error);
		if (error != NULL) {
			g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
			g_object_unref (client);
			g_object_unref (in_stream);
			g_object_unref (file_stream);
			g_object_unref (file);
			g_error_free (error);
			return FALSE;
		}
		win = gsampler_main_window_get_instance ();
		while (line != NULL) {
			gsampler_main_window_set_status_pulse (win, TRUE);
			if (strlen (line) != 0) {
				if (!g_str_has_prefix (line, "#")) {
					gchar* ret;
					ret = gsampler_client_send_commande (client, line);
					if (strlen (ret) == 0) {
						g_free (ret);
						g_free (line);
						break;
					}
					if (string_contains (ret, "ERR:")) {
						g_warning ("erreur : %s", ret);
						g_free (ret);
						g_free (line);
						break;
					}
					while (gtk_events_pending ()) { gtk_main_iteration (); }
					g_free (ret);
					g_free (line);
				}
			}
			line = g_data_input_stream_read_line (in_stream, NULL, NULL, &error);
			if (error != NULL) {
				g_warning ("file %s: line %d: error: %s", __FILE__, __LINE__, error->message);
				break;
			}
		}
		gsampler_main_window_set_status_pulse (win, FALSE);
		gsampler_session_set_is_updated (self, FALSE);
		gsampler_main_window_set_wait_cursor (FALSE);
		g_object_unref (win);
		g_object_unref (client);
		g_object_unref (in_stream);
		g_object_unref (file_stream);
		g_object_unref (file);
	}
	return FALSE;
}

static void 
gsampler_session_on_session_updated (GsamplerClient* client, gpointer data) {
	GsamplerSession* self = GSAMPLER_SESSION (data);
	g_return_if_fail (self != NULL);

	if (g_strcmp0 (self->priv->_name, "") != 0) {
		gsampler_session_set_is_updated (self, TRUE);
	}
} 

static void 
gsampler_session_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GsamplerSession * self;
	self = GSAMPLER_SESSION (object);
	switch (property_id) {
		case GSAMPLER_SESSION_NAME:
			g_value_set_string (value, gsampler_session_get_name (self));
			break;
		case GSAMPLER_SESSION_FILENAME:
			g_value_set_string (value, gsampler_session_get_filename (self));
			break;
		case GSAMPLER_SESSION_IS_UPDATED:
			g_value_set_boolean (value, gsampler_session_get_is_updated (self));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
			break;
	}
}


static void 
gsampler_session_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GsamplerSession * self;
	self = GSAMPLER_SESSION (object);
	switch (property_id) {
		case GSAMPLER_SESSION_NAME:
			gsampler_session_set_name (self, g_value_get_string (value));
			break;
		case GSAMPLER_SESSION_FILENAME:
			gsampler_session_set_filename (self, g_value_get_string (value));
			break;
		case GSAMPLER_SESSION_IS_UPDATED:
			gsampler_session_set_is_updated (self, g_value_get_boolean (value));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
			break;
	}
}

static GObject * 
gsampler_session_constructor (GType type, guint n_construct_properties, 
                                 GObjectConstructParam * construct_properties) {
	GObject * obj;
	GsamplerSession * self = NULL;
	GsamplerClient* client;
									 
	obj = G_OBJECT_CLASS (gsampler_session_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_SESSION (obj);
	client = gsampler_client_get_instance ();
	g_signal_connect_object (client, "session-updated",
	                         (GCallback) gsampler_session_on_session_updated, self, 0);
	g_object_unref (client);
	return obj;
}

static void gsampler_session_init (GsamplerSession * self) {
	self->priv = GSAMPLER_SESSION_GET_PRIVATE (self);
	self->priv->_name = g_strdup ("untitled");
	self->priv->_filename = g_strdup ("");
	self->priv->_is_updated = FALSE;
}

static void gsampler_session_finalize (GObject* obj) {
	GsamplerSession * self;
	self = GSAMPLER_SESSION (obj);
	g_free (self->priv->_name);
	g_free (self->priv->_filename);
	G_OBJECT_CLASS (gsampler_session_parent_class)->finalize (obj);
}

static void gsampler_session_class_init (GsamplerSessionClass * klass) {
	gsampler_session_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerSessionPrivate));
	G_OBJECT_CLASS (klass)->get_property = gsampler_session_get_property;
	G_OBJECT_CLASS (klass)->set_property = gsampler_session_set_property;
	G_OBJECT_CLASS (klass)->finalize = gsampler_session_finalize;
	G_OBJECT_CLASS (klass)->constructor = gsampler_session_constructor;
	
	ses_pops[GSAMPLER_SESSION_NAME] = g_param_spec_string ("name", "Name", "Gsampler session name",
	                                                        NULL,
	                                                        G_PARAM_READWRITE );
	g_object_class_install_property (G_OBJECT_CLASS (klass),
	                                 GSAMPLER_SESSION_NAME,
	                                 ses_pops[GSAMPLER_SESSION_NAME]);
	ses_pops[GSAMPLER_SESSION_FILENAME] = g_param_spec_string ("filename", "Filename",
	                                                             "Gsampler session filename", NULL,
	                                                             G_PARAM_READWRITE);
	g_object_class_install_property (G_OBJECT_CLASS (klass), 
	                                 GSAMPLER_SESSION_FILENAME, 
	                                 ses_pops[GSAMPLER_SESSION_FILENAME]);
	ses_pops[GSAMPLER_SESSION_IS_UPDATED] = g_param_spec_boolean ("is-updated", "Is-updated", 
	                                                                "Gsampler session is updated", FALSE,
	                                                                G_PARAM_READWRITE);
	g_object_class_install_property (G_OBJECT_CLASS (klass),
	                                 GSAMPLER_SESSION_IS_UPDATED,
	                                 ses_pops[GSAMPLER_SESSION_IS_UPDATED]);
}

GsamplerSession* 
gsampler_session_new (void) {
	GsamplerSession * session = NULL;
	session = (GsamplerSession*) g_object_new (GSAMPLER_TYPE_SESSION, NULL);
	return session;
}

void 
gsampler_session_clear (GsamplerSession* self) {
	g_return_if_fail (self != NULL);

	gsampler_session_set_name (self, "untitled");
	gsampler_session_set_filename (self, "");
	gsampler_session_set_is_updated (self, FALSE);
}


gboolean 
gsampler_session_ask_overide (GsamplerSession* self) {
	gboolean result = TRUE;
	g_return_val_if_fail (self != NULL, FALSE);

	if (self->priv->_is_updated == TRUE) {
		GsamplerMainWindow* win;
		GtkDialog* dialog;
		gint response;
		
		win = gsampler_main_window_get_instance ();
		dialog = (GtkDialog*) gtk_message_dialog_new ((GtkWindow*) win, GTK_DIALOG_MODAL,
		                                              GTK_MESSAGE_WARNING, GTK_BUTTONS_OK_CANCEL,
		                                              "%s", _("This will erase all unsaved changes ?"));
		gtk_dialog_set_default_response (dialog, (gint) GTK_RESPONSE_OK);
		response = gtk_dialog_run (dialog);
		if (response == GTK_RESPONSE_CANCEL) {
			result = FALSE;
		}
		gtk_widget_destroy ((GtkWidget*) dialog);
		g_object_unref (win);
	}
	return result;
}

gboolean 
gsampler_session_save_session (GsamplerSession* self) {
	g_return_val_if_fail (self != NULL, FALSE);

	if (strlen (self->priv->_filename) == 0 && g_strcmp0 (self->priv->_filename, "") == 0) {
		return gsampler_session_save_session_as (self);
	} else if (!gsampler_session_save_session_file (self)) {
		gsampler_main_window_send_message ("warning", _("Can't save lscp script"));
		return FALSE;
	}
	return TRUE;
}


gboolean 
gsampler_session_save_session_as (GsamplerSession* self) {
	gchar* filename = NULL;
	gboolean retour = TRUE;
	g_return_val_if_fail (self != NULL, FALSE);

	filename = gsampler_session_new_file_dialog (self, _("Save lscp script"));
	if (filename != NULL) {
		gsampler_session_set_filename (self, filename);
		if (!gsampler_session_save_session_file (self)) {
			gsampler_main_window_send_message ("warning", _("Can't save lscp script"));
			retour = FALSE;
		} else {
			retour = TRUE;
		}
		g_free (filename);
	}
	return retour;
}


gchar* gsampler_session_new_file_dialog (GsamplerSession* self, const gchar* title) {
	GsamplerMainWindow* win;
	GtkDialog* dialog;
	GtkFileFilter* filter;
	GsamplerOption* option;
	gchar* rep;
	gint response;
	gchar* name = NULL;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (title != NULL, NULL);
	
	win = gsampler_main_window_get_instance ();
	dialog = (GtkDialog*) gtk_file_chooser_dialog_new (title, (GtkWindow*) win,
	                                                   GTK_FILE_CHOOSER_ACTION_SAVE,
	                                                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                                   GTK_STOCK_SAVE, GTK_RESPONSE_OK,
	                                                   NULL, NULL);
	filter = gtk_file_filter_new ();
	gtk_buildable_set_name ((GtkBuildable*) filter, "lscp");
	gtk_file_filter_add_pattern (filter, "*.lscp");
	gtk_file_chooser_set_filter ((GtkFileChooser*) dialog, filter);
	option = gsampler_option_get_instance ();
	rep = gsampler_option_get_string (option, "Client", "script-path");
	gtk_file_chooser_set_current_folder ((GtkFileChooser*) dialog, rep);
	gtk_file_chooser_set_current_name ((GtkFileChooser*) dialog, ".lscp");
	response = gtk_dialog_run (dialog);
	if (response == GTK_RESPONSE_OK) 
		name = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog);
	gtk_widget_destroy ((GtkWidget*) dialog);
	g_free (rep);
	g_object_unref (option);
	g_object_unref (win);
	return name;
}

void 
gsampler_session_open_session (GsamplerSession* self) {
	GsamplerMainWindow* win;
	GtkDialog* dialog;
	GtkFileFilter* filter;
	GsamplerOption* option;
	gchar* rep;
	gint response;
	g_return_if_fail (self != NULL);
	win = gsampler_main_window_get_instance ();
	dialog = (GtkDialog*) gtk_file_chooser_dialog_new (_("Open lscp script"), (GtkWindow*) win,
	                                                   GTK_FILE_CHOOSER_ACTION_OPEN, 
	                                                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
	                                                   GTK_STOCK_OPEN, GTK_RESPONSE_OK, 
	                                                   NULL, NULL);
	filter = gtk_file_filter_new ();
	gtk_buildable_set_name ((GtkBuildable*) filter, "lscp");
	gtk_file_filter_add_pattern (filter, "*.lscp");
	gtk_file_chooser_set_filter ((GtkFileChooser*) dialog, filter);
	option = gsampler_option_get_instance ();
	rep = gsampler_option_get_string (option, "Client", "script-path");
	gtk_file_chooser_set_current_folder ((GtkFileChooser*) dialog, rep);
	response = gtk_dialog_run (dialog);
	switch (response) {
		case GTK_RESPONSE_OK:
		{
			gchar* open = NULL;
			open = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog);
			if (open != NULL) {
				gsampler_session_set_filename (self, open);
				gsampler_session_set_name (self, gsampler_utils_get_name (self->priv->_filename));
				gdk_threads_add_idle_full (G_PRIORITY_HIGH_IDLE, gsampler_session_open_session_file, 
				                           g_object_ref (self), g_object_unref);
				g_free (open);
			}
			break;
		}
		default:
		{
			break;
		}
	}
	gtk_widget_destroy ((GtkWidget*) dialog);
	g_free (rep);
	g_object_unref (option);
	g_object_unref (win);
}